@@ -337,6 +337,14 @@ def get_parser(fc_symmetry=False, is_nac=False, load_phono3py_yaml=False):
         default=None,
         help="Fixed grid addresses where anharmonic properties are calculated",
     )
+    parser.add_argument(
+        "--gm",
+        "--grid-matrix",
+        nargs="+",
+        dest="grid_matrix",
+        default=None,
+        help="Grid generating matrix for generalized regular grid",
+    )
     parser.add_argument(
         "--gp",
         "--grid-points",
@@ -749,6 +757,13 @@ def get_parser(fc_symmetry=False, is_nac=False, load_phono3py_yaml=False):
             "mesh numbers to ir_grid_address.yaml"
         ),
     )
+    parser.add_argument(
+        "--wigner",
+        dest="is_wigner_kappa",
+        action="store_true",
+        default=False,
+        help="Choose Wigner lattice thermal conductivity.",
+    )
     parser.add_argument(
         "--write-collision",
         dest="write_collision",
