@@ -43,6 +43,7 @@ class Phono3pySettings(Settings):
     _default = {
         # In micrometre. The default value is just set to avoid divergence.
         "boundary_mfp": 1.0e6,
+        "conductivity_type": None,
         "constant_averaged_pp_interaction": None,
         "create_forces_fc2": None,
         "create_forces_fc3": None,
@@ -51,6 +52,7 @@ class Phono3pySettings(Settings):
         "cutoff_pair_distance": None,
         "grid_addresses": None,
         "grid_points": None,
+        "grid_matrix": None,
         "ion_clamped": False,
         "is_bterta": False,
         "is_compact_fc": False,
@@ -113,6 +115,10 @@ class Phono3pySettings(Settings):
         """Set boundary_mfp."""
         self._v["boundary_mfp"] = val
 
+    def set_conductivity_type(self, val):
+        """Set conductivity_type."""
+        self._v["conductivity_type"] = val
+
     def set_constant_averaged_pp_interaction(self, val):
         """Set constant_averaged_pp_interaction."""
         self._v["constant_averaged_pp_interaction"] = val
@@ -149,6 +155,10 @@ class Phono3pySettings(Settings):
         """Set grid_points."""
         self._v["grid_points"] = val
 
+    # def set_grid_matrix(self, val):
+    #     """Set grid_matrix."""
+    #     self._v["grid_matrix"] = val
+
     def set_ion_clamped(self, val):
         """Set ion_clamped."""
         self._v["ion_clamped"] = val
@@ -391,6 +401,10 @@ class Phono3pyConfParser(ConfParser):
             if self._args.grid_points is not None:
                 self._confs["grid_points"] = " ".join(self._args.grid_points)
 
+        if "grid_matrix" in self._args:
+            if self._args.grid_matrix is not None:
+                self._confs["grid_matrix"] = " ".join(self._args.grid_matrix)
+
         if "ion_clamped" in self._args:
             if self._args.ion_clamped:
                 self._confs["ion_clamped"] = ".true."
@@ -467,6 +481,10 @@ class Phono3pyConfParser(ConfParser):
             if self._args.is_tetrahedron_method:
                 self._confs["tetrahedron"] = ".true."
 
+        if "is_wigner_kappa" in self._args:
+            if self._args.is_wigner_kappa:
+                self._confs["conductivity_type"] = "wigner"
+
         if "lapack_zheev_uplo" in self._args:
             if self._args.lapack_zheev_uplo is not None:
                 self._confs["lapack_zheev_uplo"] = self._args.lapack_zheev_uplo
@@ -620,6 +638,7 @@ class Phono3pyConfParser(ConfParser):
             # float
             if conf_key in (
                 "boundary_mfp",
+                "const_ave_pp",
                 "cutoff_fc3_distance",
                 "cutoff_pair_distance",
                 "max_freepath",
@@ -630,9 +649,17 @@ class Phono3pyConfParser(ConfParser):
                 self.set_parameter(conf_key, float(confs[conf_key]))
 
             # int
-            if conf_key in ("pinv_solver", "num_points_in_batch"):
+            if conf_key in (
+                "pinv_solver",
+                "num_points_in_batch",
+                "scattering_event_class",
+            ):
                 self.set_parameter(conf_key, int(confs[conf_key]))
 
+            # string
+            if conf_key in ("conductivity_type", "create_forces_fc3_file"):
+                self.set_parameter(conf_key, confs[conf_key])
+
             # specials
             if conf_key in ("create_forces_fc2", "create_forces_fc3"):
                 if type(confs[conf_key]) is str:
@@ -641,11 +668,6 @@ class Phono3pyConfParser(ConfParser):
                     fnames = confs[conf_key]
                 self.set_parameter(conf_key, fnames)
 
-            if conf_key == "create_forces_fc3_file":
-                self.set_parameter(
-                    "create_forces_fc3_file", confs["create_forces_fc3_file"]
-                )
-
             if conf_key == "dim_fc2":
                 matrix = [int(x) for x in confs["dim_fc2"].split()]
                 if len(matrix) == 9:
@@ -665,9 +687,6 @@ class Phono3pyConfParser(ConfParser):
                     else:
                         self.set_parameter("dim_fc2", matrix)
 
-            if conf_key in ("constant_averaged_pp_interaction" "const_ave_pp"):
-                self.set_parameter("const_ave_pp", float(confs["const_ave_pp"]))
-
             if conf_key == "grid_addresses":
                 vals = [
                     int(x) for x in confs["grid_addresses"].replace(",", " ").split()
@@ -681,6 +700,13 @@ class Phono3pyConfParser(ConfParser):
                 vals = [int(x) for x in confs["grid_points"].replace(",", " ").split()]
                 self.set_parameter("grid_points", vals)
 
+            if conf_key == "grid_matrix":
+                vals = [int(x) for x in confs["grid_matrix"].replace(",", " ").split()]
+                if len(vals) == 9:
+                    self.set_parameter("grid_matrix", np.reshape(vals, (3, 3)))
+                else:
+                    self.setting_error("Grid matrix are incorrectly set.")
+
             if conf_key == "lapack_zheev_uplo":
                 self.set_parameter(
                     "lapack_zheev_uplo", confs["lapack_zheev_uplo"].upper()
@@ -700,11 +726,6 @@ class Phono3pyConfParser(ConfParser):
                     vals = [int(x) for x in confs["read_collision"].split()]
                     self.set_parameter("read_collision", vals)
 
-            if conf_key == "scattering_event_class":
-                self.set_parameter(
-                    "scattering_event_class", int(confs["scattering_event_class"])
-                )
-
     def _set_settings(self):
         self.set_settings()
         params = self._parameters
@@ -721,6 +742,10 @@ class Phono3pyConfParser(ConfParser):
         if "bterta" in params:
             self._settings.set_is_bterta(params["bterta"])
 
+        # Choice of thermal conductivity type
+        if "conductivity_type" in params:
+            self._settings.set_conductivity_type(params["conductivity_type"])
+
         # Solve collective phonons
         if "collective_phonon" in params:
             self._settings.set_solve_collective_phonon(params["collective_phonon"])
@@ -765,6 +790,10 @@ class Phono3pyConfParser(ConfParser):
         if "grid_points" in params:
             self._settings.set_grid_points(params["grid_points"])
 
+        # Grid matrix
+        if "grid_matrix" in params:
+            self._settings.set_mesh_numbers(params["grid_matrix"])
+
         # Atoms are clamped under applied strain in Gruneisen parameter
         # calculation.
         if "ion_clamped" in params:
