@@ -8,6 +8,7 @@ from phono3py.phonon.grid import (
     _get_grid_points_by_bz_rotations_c,
     _get_grid_points_by_bz_rotations_py,
     _get_grid_points_by_rotations,
+    can_use_std_lattice,
     get_grid_point_from_address,
     get_grid_point_from_address_py,
 )
@@ -1152,3 +1153,42 @@ def test_get_grid_points_by_bz_rotations(si_pbesol_111):
 
     # for gps in bzgps.reshape(-1, 12):
     #     print("".join(["%d, " % gp for gp in gps]))
+
+
+def test_can_use_std_lattice():
+    """Test of can_use_std_lattice."""
+    conv_lat = [[6.06531185, 0.0, 0.0], [0.0, 0.0, 6.06531185], [0.0, -6.06531185, 0.0]]
+    std_lattice = [
+        [6.06531185, 0.0, 0.0],
+        [0.0, 6.06531185, 0.0],
+        [0.0, 0.0, 6.06531185],
+    ]
+    tmat = [[0.0, 0.5, 0.5], [-0.5, -0.5, 0.0], [0.5, 0.0, 0.5]]
+    rotations = [
+        [[1, 0, 0], [0, 1, 0], [0, 0, 1]],
+        [[0, 1, 0], [0, 0, 1], [-1, -1, -1]],
+        [[0, 0, 1], [-1, -1, -1], [1, 0, 0]],
+        [[-1, -1, -1], [1, 0, 0], [0, 1, 0]],
+        [[-1, -1, -1], [0, 0, 1], [0, 1, 0]],
+        [[1, 0, 0], [-1, -1, -1], [0, 0, 1]],
+        [[0, 1, 0], [1, 0, 0], [-1, -1, -1]],
+        [[0, 0, 1], [0, 1, 0], [1, 0, 0]],
+        [[-1, -1, -1], [1, 0, 0], [0, 0, 1]],
+        [[1, 0, 0], [0, 1, 0], [-1, -1, -1]],
+        [[0, 1, 0], [0, 0, 1], [1, 0, 0]],
+        [[0, 0, 1], [-1, -1, -1], [0, 1, 0]],
+        [[1, 0, 0], [-1, -1, -1], [0, 1, 0]],
+        [[0, 1, 0], [1, 0, 0], [0, 0, 1]],
+        [[0, 0, 1], [0, 1, 0], [-1, -1, -1]],
+        [[-1, -1, -1], [0, 0, 1], [1, 0, 0]],
+        [[0, 1, 0], [-1, -1, -1], [0, 0, 1]],
+        [[0, 0, 1], [1, 0, 0], [-1, -1, -1]],
+        [[-1, -1, -1], [0, 1, 0], [1, 0, 0]],
+        [[1, 0, 0], [0, 0, 1], [0, 1, 0]],
+        [[0, 0, 1], [1, 0, 0], [0, 1, 0]],
+        [[-1, -1, -1], [0, 1, 0], [0, 0, 1]],
+        [[1, 0, 0], [0, 0, 1], [-1, -1, -1]],
+        [[0, 1, 0], [-1, -1, -1], [1, 0, 0]],
+    ]
+
+    assert can_use_std_lattice(conv_lat, tmat, std_lattice, rotations)
